#!/usr/bin/env python
#
# -------------------------------------------------------------------------------------
#
# Copyright (c) 2023 Skinnyhouse
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification, 
# are permitted provided that the following conditions are met:
#
#    - Redistributions of source code must retain the above copyright notice, this 
#    list of conditions and the following disclaimer.
#    - Redistributions in binary form must reproduce the above copyright notice, 
#    this list of conditions and the following disclaimer in the documentation and/or 
#    other materials provided with the distribution.
#    - Neither the name of the author nor the names of its contributors may be used 
#    to endorse or promote products derived from this software without specific prior 
#    written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
# SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
# DAMAGE.
#
# -------------------------------------------------------------------------------------
#
# This file is a basic Python plug-in for GIMP.
#
# It can be executed by selecting the menu option: 'Filters>/Custom Fonts/Utility Tools/Custom Font Folder Thumbnails v0.09
# Version 0.01 Options to create a png format thumbnail file for Custom Fonts for a named directory
# Version 0.02 Amended legend to count actual characters rather than just layers
# Version 0.03 Added kerning path count
# Version 0.04 Added non-English character count/ fill background with current gradient
# Version 0.05 Added language specific counter
# Version 0.06 changed menu location
# Version 0.07 improved accuracy of accented character count
# Version 0.08 improved accuracy of accented character count again and altered associated wording
# Version 0.09 change of menu location to utility tools
# Version 0.10 added option to use current foreground/background colours for information text


from gimpfu import *
import os
#from array import array
#import math
#import random

#from gtk import gdk
#w = gdk.screen_width()
#h = gdk.screen_height()

# Some global colours we may need to define

black = gimpcolor.RGB(0, 0, 0)
white = gimpcolor.RGB(255, 255, 255)


def do_stuff(image, layer, folder_info, text_colours, display_credit, do_layers, do_paths, lang_info, bg_col, do_drop):
    #### Main Filter Proc
	
    # Store the GIMP's settings so they can be restored
    gimp.context_push()
    # Set start undo point
    #pdb.gimp_image_undo_group_start(image)	
	
    #pdb.gimp_message(str(folder_info))
		
    display = False
    do_folder = True
	
    
		
    #filename = pdb.gimp_image_get_filename(image)
    if str(folder_info) == "None": 
        do_folder = False
        old_handler = pdb.gimp_message_get_handler()	
        pdb.gimp_message_set_handler(MESSAGE_BOX)
        pdb.gimp_message("Please select a folder to work with and try again.")
        pdb.gimp_message_set_handler(old_handler)
	
    if do_folder:
	
        dutch_case = unichr(192) + unichr(193)  + unichr(194) + unichr(196) +  unichr(200) + unichr(201) + unichr(202)  + unichr(203) # A grave + A acute + A circumflex + A diaeresis + E grave + E acute  + E circumflex + E diaeresis
        dutch_case += unichr(204) + unichr(205) + unichr(206) + unichr(207) + unichr(306)  + unichr(210) + unichr(211) + unichr(212) + unichr(214)# I grave + I acute + I circumflex + I diaeresis + ligature IJ + O grave + O acute +  O circumflex + O  diaeresis
        dutch_case += unichr(217) + unichr(218) + unichr(219) + unichr(220)  + unichr(221) + unichr(376) # U grave + U acute + U circumflex + U diaeresis + Y acute + Y diaeresis
        dutch_case += unichr(224) + unichr(225) + unichr(226) + unichr(228) + unichr(232) + unichr(233) + unichr(234) + unichr(235) #  a grave + a acute + a circumflex + a diaeresis  + e grave + e acute + e circumflex + e diaeresis
        dutch_case += unichr(236) + unichr(237) + unichr(238) + unichr(239) + unichr(307) + unichr(242) + unichr(244) + unichr(246) # i grave + i acute + i circumflex + i diaeresis + ligature ij + o grave + o circumflex + o  diaeresis
        dutch_case += unichr(249) + unichr(251) + unichr(252) + unichr(253) + unichr(255) # u grave + u circumflex  + u diaeresis + y acute + y diaeresis 
		
        french_case = unichr(192) + unichr(194) + unichr(196) + unichr(198) + unichr(199) # A grave + A circumflex + A diaeresis + AE ligature + C cedilla
        french_case +=  unichr(200) + unichr(201) + unichr(202) + unichr(203) + unichr(206) + unichr(207) # E grave + E acute + E circumflex + E diaeresis + I circumflex + I diaeresis
        french_case += unichr(212) + unichr(217) + unichr(219) + unichr(220) # O circumflex + U grave + U circumflex + U diaeresis
        french_case += unichr(224) + unichr(226) + unichr(228) + unichr(230) + unichr(231) # a grave + a circumflex + a diaeresis + ae ligature + c cedilla
        french_case += unichr(232) + unichr(233) + unichr(234) + unichr(235) + unichr(238) + unichr(239) # e grave + e acute + e circumflex + e diaeresis + i circumflex + i diaeresis
        french_case += unichr(244) + unichr(249) + unichr(251) + unichr(252) + unichr(338) + unichr(339) + unichr(376) + unichr(255) # o circumflex + u grave + u circumflex + u diaeresis + OE ligature + oe ligature + Y diaeresis + y diaeresis
		
        german_case = unichr(196) + unichr(214) + unichr(220) + unichr(223) # A diaeresis + O diaeresis + U diaeresis + Eszett
        german_case += unichr(228) + unichr(246) + unichr(252) # a diaeresis + o diaeresis + u diaeresis
		
        italian_case = unichr(192) + unichr(200) + unichr(201) + unichr(204) + unichr(210) + unichr(217) #  A grave + E grave + E acute + I grave + O grave + U grave
        italian_case += unichr(224) + unichr(232) + unichr(233) + unichr(236) + unichr(242) + unichr(249) # a grave + e grave + e acute + i grave + o grave + u grave
		
        polish_case = unichr(260) + unichr(262) + unichr(280) + unichr(321) + unichr(323) + unichr(211) + unichr(346) + unichr(377) + unichr(379) # A ogonek + C acute + E ogonek + L stroke + N acute + O acute + S acute + Z acute + Z dot above
        polish_case += unichr(261) + unichr(263) + unichr(281) + unichr(322) + unichr(324) + unichr(243) + unichr(347) + unichr(378) + unichr(380) # a ogonek + c acute + e ogonek + l stroke + n acute + o acute + s acute + z acute + z dot above
		
        portuguese_case =  unichr(192) + unichr(193) + unichr(194) + unichr(195) + unichr(201) + unichr(202) + unichr(205) + unichr(211) + unichr(212) + unichr(213) # A grave + A acute + A circumflex + A tilde + E acute + E circumflex + I acute + O acute + O circumflex + O tilde
        portuguese_case += unichr(218) + unichr(199) + unichr(220) + unichr(224) + unichr(225) + unichr(226) + unichr(227) + unichr(233) + unichr(234) # U Acute + C cedilla + U diaeresis + a grave + a acute + a circumflex + a tilde + e acute + e circumflex
        portuguese_case += unichr(237) + unichr(243) + unichr(244) + unichr(245) + unichr(250)  + unichr(231) + unichr(252) # i acute + o acute + o circumflex + o tilde + u acute + c cedilla + u diaeresis
		
        spanish_case = unichr(161) + unichr(191) + unichr(193) + unichr(201) + unichr(205) + unichr(209) + unichr(211) # inverted exclamation + inverted question + A acute + E acute + I acute + N tilde + O acute
        spanish_case += unichr(218) + unichr(220) + unichr(225) + unichr(233) + unichr(237) + unichr(241) # U Acute + U diaeresis + a acute + e acute + i acute + n tilde
        spanish_case += unichr(243) + unichr(250) + unichr(252)# o acute + u acute + u diaeresis
		
        non_english = dutch_case + french_case + german_case + italian_case + polish_case + portuguese_case + spanish_case
		
        other_languages = False
		
        new_image = pdb.gimp_image_new(600, 600, 0)
		
        # Set start undo point
        pdb.gimp_image_undo_group_start(new_image)
		
        bg_layer = pdb.gimp_layer_new(new_image, 600, 600, 1, "background layer", 100, 28)
        pdb.gimp_image_insert_layer(new_image, bg_layer, None, -1)
        display = pdb.gimp_display_new(new_image)
		
		
        #filename = pdb.gimp_image_get_filename(image) # get information about the image file name and file path
        #file_extension = os.path.splitext(filename)[1]
        #file_path = os.path.splitext(filename)[0]
        #base_path = os.path.basename(filename)
		
        temp = 0
			
        for file in os.listdir(folder_info):
            file_extension = os.path.splitext(file)[1]
            file_name = os.path.join(folder_info, file)
            file_path = os.path.splitext(file_name)[0]
            base_path = os.path.basename(file_name)
			
            if file_extension.lower() == ".xcf":
                font_name_pointer, file_dummy = -1, 0
                #pdb.gimp_message("file = " + str(file_name))
                #pdb.gimp_message("folder = " + str(folder_info))
                fontimg = pdb.gimp_xcf_load(file_dummy,file_name,file_name)
                num_layers, layer_ids = pdb.gimp_image_get_layers(fontimg)
                layer_names_list = fontimg.layers
                #pdb.gimp_message("Number of layers = " + str(num_layers))
                #count the number of kerning paths in the file
                num_paths, vector_ids = pdb.gimp_image_get_vectors(fontimg)
				
                loop, char_ctr, non_en, dutch, french, german, italian, polish, portuguese, spanish = 0,0,0,0,0,0,0,0,0,0 # count non-English characters
                while loop < num_layers:
                    char_name = pdb.gimp_item_get_name(layer_names_list[loop])
                    char_name = char_name.decode('utf-8')
                    char = char_name[0]
					
                    if (len(char_name) == 5 and char_name.find(".png") == 1) or (len(char_name) == 6 and char_name.find(".png") == 2):
                        if lang_info == 1:
                            if char in dutch_case: 
                                other_languages = True
                                non_en += 1
                                dutch += 1
                            elif char in french_case: 
                                other_languages = True
                                non_en += 1
                                french += 1
                            elif char in german_case: 
                                other_languages = True
                                non_en += 1
                                german += 1
                            elif char in italian_case: 
                                other_languages = True
                                non_en += 1
                                italian += 1
                            elif char in polish_case: 
                                other_languages = True
                                non_en += 1
                                polish += 1
                            elif char in portuguese_case: 
                                other_languages = True
                                non_en += 1
                                portuguese += 1
                            elif char in spanish_case: 
                                other_languages = True
                                non_en += 1
                                spanish += 1
						
                        else:
                            if char in dutch_case: 
                                other_languages = True
                                non_en += 1
                                dutch += 1
                            if char in french_case: 
                                other_languages = True
                                non_en += 1
                                french += 1
                            if char in german_case: 
                                other_languages = True
                                non_en += 1
                                german += 1
                            if char in italian_case: 
                                other_languages = True
                                non_en += 1
                                italian += 1
                            if char in polish_case: 
                                other_languages = True
                                non_en += 1
                                polish += 1
                            if char in portuguese_case: 
                                other_languages = True
                                non_en += 1
                                portuguese += 1
                            if char in spanish_case: 
                                other_languages = True
                                non_en += 1
                                spanish += 1
                        char_ctr += 1
                    loop += 1				
				
				
                vector_list = fontimg.vectors
                loop, kern_ctr = 0,0
                while loop < num_paths:
                    path_name = pdb.gimp_item_get_name(vector_list[loop])
                    path_name = path_name.decode('utf-8')
                    if (len(path_name) == 5 and path_name.find(".png") == 1) or (len(path_name) == 6 and path_name.find(".png") == 2) or len(path_name) == 2:
                        kern_ctr += 1
                    loop += 1				
				
				
                copy = pdb.gimp_image_get_layer_by_name(fontimg,"A.png")
				
                if not copy: copy = pdb.gimp_image_get_layer_by_name(fontimg,"a.png")
				
                if copy:
                    
                    #pdb.gimp_message("File = " + str(temp))
                    display_copy = pdb.gimp_layer_new_from_drawable(copy,new_image)
                    pdb.gimp_image_insert_layer(new_image, display_copy, None, -1)
                    # Rename the new layer
                    pdb.gimp_item_set_name(display_copy,"Display")
                    # turn on visibility of layer
                    pdb.gimp_item_set_visible(display_copy, True)
			
                    display_offset_x = display_copy.width # make a note of the width of the uppercase "A" layer
                    display_offset_y = display_copy.height # make a note of the height of the uppercase "A" layer
					
                    pdb.gimp_image_resize(new_image, display_offset_x, display_offset_y, 0, 0)
					
                    copy2 = pdb.gimp_image_get_layer_by_name(fontimg,"b.png") # Make a copy of lowercase "b" if present in the stack
                    if copy2:
                        display_copy2 = pdb.gimp_layer_new_from_drawable(copy2,new_image)
                        pdb.gimp_image_insert_layer(new_image, display_copy2, None, -1)
                    else:
                        copy2 = pdb.gimp_image_get_layer_by_name(fontimg,"B.png") # if lowercase "b" is not present use uppercase "B" or uppercase "A"
                        if copy2:
                            display_copy2 = pdb.gimp_layer_new_from_drawable(copy2,new_image)
                            pdb.gimp_image_insert_layer(new_image, display_copy2, None, -1)
                        else:
                            display_copy2 = pdb.gimp_layer_new_from_drawable(copy,new_image)
                            pdb.gimp_image_insert_layer(new_image, display_copy2, None, -1)

                    pdb.gimp_layer_set_offsets(display_copy2, display_offset_x, 0) # position the second display character layer
                    # turn on visibility of layer
                    pdb.gimp_item_set_visible(display_copy2, True)
                    # merge the two characters
                    display_id = pdb.gimp_image_merge_down(new_image, display_copy2, 0)
                    # resize the image if required
                    if display_id.width > new_image.width: pdb.gimp_image_resize(new_image, display_id.width, new_image.height, 0, 0)
					
                    # Remove any unnecessary space below the character display
                    # Copy and insert new layer from display layer
                    copy3 = pdb.gimp_layer_new_from_drawable(display_id,new_image)
                    pdb.gimp_image_insert_layer(new_image, copy3, None, -1)
                    pdb.gimp_item_set_visible(copy3, True)
                    # crop empty borders
                    pdb.plug_in_autocrop_layer(new_image, copy3)
                    # get offset information
                    cropped_x_offset, cropped_y_offset = pdb.gimp_drawable_offsets(copy3)
                    #pdb.gimp_message("cropped_y_offset = " + str(cropped_y_offset) )
                    #if cropped_y_offset > 12: cropped_y_offset = 12 # set maximum space above display layer content
                    #text_layer_y_offset = cropped_y_offset + copy3.height + 20 # set y offset for text layer
                    text_layer_y_offset = cropped_y_offset + copy3.height # set y offset for text layer
                    #pdb.gimp_image_resize(new_image, display_id.width, (cropped_y_offset + copy3.height+20), 0, 0)
                    #y_offset = cropped_y_offset + copy3.height+20
                    pdb.gimp_image_resize(new_image, display_id.width, (cropped_y_offset + copy3.height), 0, 0)
                    y_offset = cropped_y_offset + copy3.height
                    pdb.gimp_layer_resize(display_id, display_id.width, y_offset, 0, 0)
                    pdb.gimp_image_remove_layer(new_image, copy3)
                    

                    # get the text to put into the new text layer
                    our_text = base_path
                    if str(display_credit) <> "": our_text += "\n" + display_credit
                    if do_layers:
                        our_text += "\n" + str(char_ctr) + " characters."
                    if do_paths:
                        if kern_ctr == 1:
                            our_text += "\n" + str(kern_ctr) + " kerning path."
                        else:
                            our_text += "\n" + str(kern_ctr) + " kerning paths."
                    if lang_info>0:
                        if lang_info==1:
                            if other_languages: 
                                if non_en == 1:
                                    our_text += "\n" + "Contains just 1 non-English character."
                                else:
                                    our_text += "\n" + "Contains " + str(non_en) + " non-English characters."
                        else:
                            if dutch>0:
                                if dutch == 1:
                                    our_text += "\n" + "Contains 1 accented character for use in Dutch."
                                else:
                                    our_text += "\n" + "Contains " + str(dutch) + " accented characters for use in Dutch."
                            if french>0:
                                if french == 1:
                                    our_text += "\n" + "Contains 1 accented character for use in French."
                                else:
                                    our_text += "\n" + "Contains " + str(french) + " accented characters for use in French."
                            if german>0:
                                if german == 1:
                                    our_text += "\n" + "Contains 1 accented character for use in German."
                                else:
                                    our_text += "\n" + "Contains " + str(german) + " accented characters for use in German."
                            if italian>0:
                                if italian == 1:
                                    our_text += "\n" + "Contains 1 accented character for use in Italian."
                                else:
                                    our_text += "\n" + "Contains " + str(italian) + " accented characters for use in Italian."
                            if polish>0:
                                if polish == 1:
                                    our_text += "\n" + "Contains 1 accented character for use in Polish."
                                else:
                                    our_text += "\n" + "Contains " + str(polish) + " accented characters for use in Polish."
                            if portuguese>0:
                                if portuguese == 1:
                                    our_text += "\n" + "Contains 1 accented character for use in Portuguese."
                                else:
                                    our_text += "\n" + "Contains " + str(portuguese) + " accented characters for use in Portuguese."
                            if spanish>0:
                                if spanish == 1:
                                    our_text += "\n" + "Contains 1 accented character for use in Spanish."
                                else:
                                    our_text += "\n" + "Contains " + str(spanish) + " accented characters for use in Spanish."
                    # add the new text layer to the layer stack
                    text_layer = pdb.gimp_text_layer_new(new_image,our_text, "Sen", 24, 0)
                    pdb.gimp_image_insert_layer(new_image, text_layer, None, -1)
                    if text_colours:
                        pdb.gimp_text_layer_set_color(text_layer, pdb.gimp_context_get_foreground())
                        
                    else:
                        pdb.gimp_text_layer_set_color(text_layer, black)
                    pdb.gimp_item_set_visible(text_layer, True)
                    # sort out positioning for text if user text is present
                    if str(display_credit) <> "": 
                        pdb.gimp_layer_set_offsets(text_layer, 8, text_layer_y_offset+8)
                    else:
                        pdb.gimp_layer_set_offsets(text_layer, 8, text_layer_y_offset+8)
						
                    
						
                    # create a copy of the text layer to fill as a text background
                    text_copy = pdb.gimp_layer_copy(text_layer, True)
                    pdb.gimp_image_insert_layer(new_image, text_copy, None, -1)
                    if text_colours:
                        pdb.gimp_drawable_fill(text_layer, 1) # background fill
                    else:
                        pdb.gimp_drawable_fill(text_layer, 2) # white fill
					
                    # merge the text layers into the display layer
					
                    updated_height = text_layer_y_offset + text_layer.height + 16
                    updated_width = display_id.width
                    pdb.gimp_image_resize(new_image, display_id.width, updated_height, 0, 0)
					
                    ##### Add drop Shadow here for display_id layer
                    if do_drop:
                        drop_offset_x, drop_offset_y, drop_blur, drop_opacity, drop_colour, drop_relief = 8, 8, 10, 80, 1, 0 # force some values
                        display_id = proc_dropshadow(new_image, display_id, display_id, drop_offset_x, drop_offset_y, drop_blur, drop_opacity, drop_colour, drop_relief)
					
					
                    text_copy = pdb.gimp_image_merge_down(new_image, text_copy, 0)
                    display_id = pdb.gimp_image_merge_down(new_image, text_copy, 0)
					
                    pdb.gimp_layer_set_offsets(display_id, 0, 0)
                    pdb.gimp_image_resize(new_image, display_id.width, updated_height, 0, 0)
                    pdb.gimp_layer_resize_to_image_size(display_id)
                    copy4 = pdb.gimp_layer_new_from_drawable(display_id,new_image)
                    pdb.gimp_image_insert_layer(new_image, copy4, None, -1)
                    pdb.gimp_item_set_visible(copy4, True)
                    pdb.gimp_drawable_edit_clear(copy4)
                    pdb.gimp_drawable_fill(copy4, bg_col) # fill
                    if bg_col == 2: # use trick transparency
                        pdb.gimp_layer_set_opacity(copy4, 1)
                    if bg_col == 3: # use current gradient
                        pdb.gimp_drawable_edit_gradient_fill(copy4, 0, 0, False, 1, 0, True, 0, 0, 0, copy4.height)
                    pdb.gimp_image_lower_item(new_image, copy4)
                    display_id = pdb.gimp_image_merge_down(new_image, display_id, 0)
                    #pdb.gimp_message("updated_height = " + str(updated_height))
                    #pdb.gimp_image_crop(new_image, updated_width, updated_height, 0, 0)
                    #sausage = mash
                    do_save_png = True
                    if do_save_png:
                        # save a png file without asking the user
                        save_file = file_path + ".png"
                        pdb.file_png_save2(image, display_id, save_file, save_file, False, 9, True, False, False, True, True, True, True)
					
                    #pdb.gimp_image_remove_layer(new_image, display_id)
                    temp += 1
		
        pdb.gimp_layer_resize_to_image_size(bg_layer)
		
        # Set end undo point
        pdb.gimp_image_undo_group_end(new_image)
    if display: pdb.gimp_display_delete(display)
		

    # Restore the GIMP's settings
    gimp.context_pop()
	
######################################################################################################################################

    #### Useful Functions
# -------------------------------------------------------------------------------------

######################################################################################################################################
	
def proc_messagebox(message):
    old_handler = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(2)
    pdb.gimp_message(str(message))
    pdb.gimp_message_set_handler(old_handler)
	
######################################################################################################################################

def proc_dropshadow(im, original_layer, target_layer, drop_offset_x, drop_offset_y, drop_blur, drop_opacity, drop_colour, drop_relief):
    #REM Create a text drop shadow effect - optionally clipped to the text layer for transparent type texts
    pdb.gimp_image_freeze_layers(im)
    old_fg = pdb.gimp_context_get_foreground() # added to cater for thumbnail use
    pdb.gimp_context_set_foreground(black) # added to cater for thumbnail use
    # make a copy of the image width/height
    old_im_width, old_im_height = im.width, im.height
    name = pdb.gimp_item_get_name(target_layer)
    # Copy layer to use as the drop shadow
    shadow_layer = pdb.gimp_layer_copy(target_layer, False)
    # Add and re-position the layer in the layer stack
    im.add_layer(shadow_layer, -1)
    pdb.gimp_image_lower_item(im, shadow_layer)
    pdb.gimp_image_lower_item(im, shadow_layer) # added to cater for thumbnail use
    pdb.gimp_image_lower_item(im, shadow_layer) # added to cater for thumbnail use
    # get layer offsets for the shadow layer (which will also be the same as the original layer holding the user text)
    shadow_offset_x, shadow_offset_y = pdb.gimp_drawable_offsets(shadow_layer)
    # work out the centre point of the shadow/original layer
    shad_centre_x = (shadow_layer.width/2)
    shad_centre_y = (shadow_layer.height/2)
    # make up new size for the shadow layer to accommodate blurred result - currently twice the existing width and height
    new_shad_width = shadow_layer.width*2
    new_shad_height = shadow_layer.height*2
    # work out new offsets for the new size of the shadow layer to ensure that the shadow and original layers are still aligned
    new_shad_layer_offset_x = 0 - ((shadow_layer.width - new_shad_width)/2)
    new_shad_layer_offset_y = 0 - ((shadow_layer.height - new_shad_height)/2)
    # and apply the new offsets to the shadow layer
    pdb.gimp_layer_resize(shadow_layer, new_shad_width, new_shad_height, new_shad_layer_offset_x, new_shad_layer_offset_y)
    # make a note of the new shadow layer offsets
    shadow_resized_offset_x, shadow_resized_offset_y = pdb.gimp_drawable_offsets(shadow_layer)	
	
    # if any of the shadow layer falls outside the image boundaries
    reset_shadow = False
    if new_shad_layer_offset_x<0 or new_shad_layer_offset_y<0 or shadow_layer.width > im.width or shadow_layer.height > im.height:
        reset_shadow = True
        # set offsets of the shadow layer to image origin
        pdb.gimp_layer_set_offsets(shadow_layer,0,0)
        # set offsets for original text layer to match
        pdb.gimp_layer_set_offsets(target_layer,0 + abs(new_shad_layer_offset_x) ,0 + abs(new_shad_layer_offset_y))
        # resize the image to accomodate the new shadow layer size
        new_im_width, new_im_height = im.width, im.height
        if shadow_layer.width > im.width: new_im_width = shadow_layer.width
        if shadow_layer.height > im.height: new_im_height = shadow_layer.height
        pdb.gimp_image_resize(im, new_im_width, new_im_height, 0, 0)
		
    # get alpha selection
    pdb.gimp_image_select_item(im, 2, shadow_layer)
    if drop_offset_x==0 and drop_offset_y==0 and drop_relief>0: pdb.gimp_selection_grow(im, drop_relief)
    # fill the text on the shadow layer with colour
    if drop_colour<3:
        pdb.gimp_drawable_edit_fill(shadow_layer,drop_colour-1)
    # deselect
    pdb.gimp_selection_none(im)
    #pdb.gimp_image_select_item(im, 2, original_layer)
    # blur shadow 
    pdb.plug_in_gauss(im, shadow_layer, drop_blur, drop_blur, 1)
    #pdb.gimp_selection_none(im)
    # crop the blurred layer
    pdb.plug_in_autocrop_layer(im, shadow_layer)
    # move the shadow layer to user position and adjust opacity
    temp = pdb.gimp_layer_translate(shadow_layer, drop_offset_x, drop_offset_y)
    pdb.gimp_layer_set_opacity(shadow_layer, drop_opacity)
    # merge and rename
    target_layer = pdb.gimp_image_merge_down(im,target_layer,0)
    pdb.gimp_item_set_name(target_layer, name)
    if reset_shadow: 
        cropped_offset_x, cropped_offset_y = pdb.gimp_drawable_offsets(target_layer)	
        pdb.gimp_layer_set_offsets(target_layer,shadow_offset_x, shadow_offset_y)
    pdb.gimp_image_resize(im, old_im_width, old_im_height, 0, 0)
    
    pdb.gimp_context_set_foreground(old_fg) # added to cater for thumbnail use
    pdb.gimp_image_thaw_layers(im)
    return target_layer
    
######################################################################################################################################

######################################################################################################################################
	
register(
    "python_fu_Custom_Font_Folder_Thumbnails_v0-10",
    "Provides option for adding image thumbnails to all custom font files in a designated folder",
    "Provides options for adding image thumbnails to all custom font files in a designated folder",
    "Skinnyhouse",
    "Open source (BSD 3-clause license)",
    "2023",
    "<Image>/Filters/Custom Fonts/Utility Tools/Custom Font Folder Thumbnails v0.10...",
    "*",
    [
    (PF_DIRNAME, "pf_folder", "Folder to use:", 0),
    (PF_TOGGLE, "pf_Text_colours", "Use current foreground and background colours in text box: ", 0),	
    (PF_STRING, "pf_credit", "Display credit: ", "Decorated by a Gimp Lover."),
    (PF_TOGGLE, "pf_Add_Layer_Info", "Include custom font layers count: ", 1),
    (PF_TOGGLE, "pf_Add_Kerning_Info", "Include custom font kerning paths count: ", 1),
    (PF_OPTION, "lang_option", "Non-English information: ", 0, ("No information", "Overall information", "Language specific")),
    (PF_OPTION, "colour_option", "Background colour options: ", 2, ("Use current foreground colour as background", "Use current background colour as background", "Use a transparent background", "Use current gradient: ")),
    (PF_TOGGLE, "pf_Add_drop", "Add black drop shadow : ", 1),
    ],
    [],
    do_stuff)

main()
